@@ -1,6 +1,6 @@
 ;;; psgml-edit.el --- Editing commands for SGML-mode with parsing support
 ;;
-;; $Id: psgml-edit.el,v 2.65 2002/02/15 15:31:47 lenst Exp $
+;; $Id: psgml-edit.el,v 2.69 2002/08/13 15:13:23 lenst Exp $
 
 ;; Copyright (C) 1994, 1995, 1996 Lennart Staflin
 
@@ -31,9 +31,10 @@
 (provide 'psgml-edit)
 (require 'psgml)
 (require 'psgml-parse)
+(eval-when-compile (require 'cl))
 
-(eval-when-compile
-  (setq byte-compile-warnings '(free-vars unresolved callargs redefine)))
+;; (eval-when-compile
+;;   (setq byte-compile-warnings '(free-vars unresolved callargs redefine)))
 
 
 ;;;; Variables
@@ -171,7 +172,11 @@ possible."
     (error "Illegal name"))
   (let* ((element (sgml-find-element-of (point)))
 	 (attspec (sgml-element-attribute-specification-list element))
-	 (oldattlist (sgml-element-attlist element)))
+	 (oldattlist (sgml-element-attlist element))
+         (tagc (if (and sgml-xml-p (sgml-element-empty element))
+                (sgml-delim "XML-TAGCE")
+              (sgml-delim "TAGC")))
+         (tagc-len (length tagc)))
     (goto-char (sgml-element-end element))
     (unless  (sgml-element-empty element)
       (delete-char (- (sgml-element-etag-len element))))
@@ -179,15 +184,15 @@ possible."
     (goto-char (sgml-element-start element))
     (delete-char (sgml-element-stag-len element))
     (insert (sgml-delim "STAGO")
-            (sgml-general-insert-case gi))
+            (sgml-general-insert-case gi)
+            tagc)
     (let* ((newel (sgml-find-context-of (point)))
 	   (newattlist (sgml-element-attlist newel))
 	   (newasl (sgml-translate-attribute-specification-list
 		    attspec oldattlist newattlist)))
-      (sgml-insert-attributes newasl newattlist))
-    (insert (if (and sgml-xml-p (sgml-element-empty element))
-                (sgml-delim "XML-TAGCE")
-              (sgml-delim "TAGC")))))
+      (backward-char tagc-len)
+      (sgml-insert-attributes newasl newattlist)
+      (forward-char tagc-len))))
 
 
 (defun sgml-translate-attribute-specification-list (values from to)
@@ -783,7 +788,7 @@ AVL should be a assoc list mapping symbols to strings."
   (let ((quote ""))
 	(cond ((and (not sgml-always-quote-attributes)
 		    sgml-shorttag
-		    (string-match "\\`[.A-Za-z0-9---]+\\'" value))
+		    (string-match "\\`[-.A-Za-z0-9]+\\'" value))
 	       ) ; no need to quote
 	      ((not (string-match "\"" value)) ; can use "" quotes
 	       (setq quote "\""))
@@ -849,6 +854,13 @@ AVL should be a assoc list mapping symbols to strings."
 (defun sgml-change-start-tag (element asl)
   (let ((name (sgml-element-gi element))
 	(attlist (sgml-element-attlist element)))
+    ;; Concoct an attribute specification list using the names of the
+    ;; existing attributes and those ot be changed.
+    (when (and (not attlist) sgml-dtd-less)
+      (dolist (elt (mapcar 'car asl))
+	(unless (assoc elt attlist)	; avoid duplicates
+	  (push (sgml-make-attdecl elt 'CDATA 'REQUIRED) attlist)))
+      (setq attlist (nreverse attlist)))
     (assert (sgml-bpos-p (sgml-element-stag-epos element)))
     (goto-char (sgml-element-start element))
     (delete-char (sgml-element-stag-len element))
@@ -901,13 +913,18 @@ CURVALUE is nil or a string that will be used as default value."
             (let ((completion-ignore-case sgml-namecase-general))
               (completing-read
                "Attribute name: "
-               (mapcar (function (lambda (a) (list (sgml-attdecl-name a))))
-                       (sgml-non-fixed-attributes (sgml-element-attlist el)))
-               nil t)))))
+               (mapcar
+		(function (lambda (a) (list (sgml-attdecl-name a))))
+		(if sgml-dtd-less
+		    (sgml-tree-asl el)
+		  (sgml-non-fixed-attributes (sgml-element-attlist el))))
+               nil (not sgml-dtd-less))))))
      (list name
 	   (sgml-read-attribute-value
-	    (sgml-lookup-attdecl name (sgml-element-attlist el))
-			(sgml-element-name el)
+	    (if sgml-dtd-less
+		(list name)
+	      (sgml-lookup-attdecl name (sgml-element-attlist el)))
+	    (sgml-element-name el)
 	    (sgml-element-attval el name)))))
   ;; Body
   (assert (stringp name))
@@ -1057,14 +1074,23 @@ tag inserted."
 	      (nconc tab
 		     (mapcar (function sgml-start-tag-of)
 			     (sgml-current-list-of-valid-eltypes)))))))
+    (if sgml-dtd-less
+	;; The best we can do is assemble a list of elements we've
+	;; seen so far.
+	(dolist (n (append (sgml-dtd-eltypes sgml-dtd-info) '())
+		   ;; Space avoids possible clash with valid element.
+		   (setq tab (cons "Any " (cons "--" tab))))
+	  (when (and (symbolp n) (not (memq n tab)))
+	    (push (symbol-name n) tab))))
     (or tab
 	(error "No valid %s at this point" type))
-    (or
-     (sgml-popup-menu event
-		      title
-		      (mapcar (function (lambda (x) (cons x x)))
-			      tab))
-     (message nil))))
+    (let ((elt (sgml-popup-menu event
+				title
+				(mapcar (function (lambda (x) (cons x x)))
+					tab))))
+      (if (equal elt "Any ")
+	  (setq elt (sgml-read-element-name "Element: ")))
+      (or elt (message nil)))))
 
 (defun sgml-entities-menu (event)
   (interactive "*e")
@@ -1116,6 +1142,20 @@ buffers local variables list."
 
 (defun sgml-make-attrib-menu (el)
   (let ((attlist (sgml-non-fixed-attributes (sgml-element-attlist el))))
+    (if (and (not attlist) sgml-dtd-less)
+      (let ((name
+	     (sgml-general-case
+	      (let ((completion-ignore-case sgml-namecase-general))
+		(completing-read
+		 "Attribute name: "
+		 (mapcar
+		  (lambda (a) (list (sgml-attdecl-name a)))
+		  (if sgml-dtd-less
+		      (sgml-tree-asl el)
+		    (sgml-non-fixed-attributes (sgml-element-attlist el))))
+		 nil (not sgml-dtd-less))))))
+	(if name
+	    (setq attlist (list (sgml-make-attdecl name 'CDATA nil))))))
     (or attlist
 	(error "No non-fixed attributes for element"))
     (loop for attdecl in attlist
@@ -1279,7 +1319,6 @@ Editing is done in a separate window."
 	   (xml-p sgml-xml-p))
       (switch-to-buffer-other-window
        (sgml-attribute-buffer element asl))
-      (sgml-edit-attrib-mode)
       (make-local-variable 'sgml-start-attributes)
       (setq sgml-start-attributes start)
       (make-local-variable 'sgml-always-quote-attributes)
@@ -1322,12 +1361,15 @@ Editing is done in a separate window."
       (setq buf (get-buffer-create bname))
       (set-buffer buf)
       (erase-buffer)
+      (sgml-edit-attrib-mode)
       (make-local-variable 'sgml-attlist)
       (setq sgml-attlist (sgml-effective-attlist
                           (sgml-element-eltype element)))
       (sgml-insert '(read-only t)
-		   "<%s  -- Edit values and finish with C-c C-c --\n"
-		   (sgml-element-name element))
+                   (substitute-command-keys
+                    "<%s  -- Edit values and finish with \
+\\[sgml-edit-attrib-finish], abort with \\[sgml-edit-attrib-abort] --\n")
+                   (sgml-element-name element))
       (loop
        for attr in sgml-attlist do
        ;; Produce text like
@@ -1391,7 +1433,7 @@ Editing is done in a separate window."
 
 (define-key sgml-edit-attrib-mode-map "\C-c\C-c" 'sgml-edit-attrib-finish)
 (define-key sgml-edit-attrib-mode-map "\C-c\C-d" 'sgml-edit-attrib-default)
-(define-key sgml-edit-attrib-mode-map "\C-c\C-k" 'sgml-edit-attrib-clear)
+(define-key sgml-edit-attrib-mode-map "\C-c\C-k" 'sgml-edit-attrib-abort)
 
 (define-key sgml-edit-attrib-mode-map "\C-a"  'sgml-edit-attrib-field-start)
 (define-key sgml-edit-attrib-mode-map "\C-e"  'sgml-edit-attrib-field-end)
@@ -1410,6 +1452,16 @@ value.  To abort edit kill buffer (\\[kill-buffer]) and remove window
   (use-local-map sgml-edit-attrib-mode-map)
   (run-hooks 'text-mode-hook 'sgml-edit-attrib-mode-hook))
 
+(defun sgml-edit-attrib-abort ()
+  "Abort the attribute editor, removing the window."
+  (interactive)
+  (let ((cb (current-buffer))
+	(start sgml-start-attributes))
+    (delete-windows-on cb)
+    (kill-buffer cb)
+    (when (markerp start)
+      (switch-to-buffer (marker-buffer start))
+      (goto-char start))))
 
 (defun sgml-edit-attrib-finish ()
   "Finish editing and insert attribute values in original buffer."
@@ -1534,14 +1586,18 @@ value.  To abort edit kill buffer (\\[kill-buffer]) and remove window
   (interactive)
   (if (eq t (get-text-property (point) 'read-only))
       (beginning-of-line 1))
-  (or (search-forward-regexp "^ *[_.:A-Za-z0-9---]+ *= ?" nil t)
+  (or (search-forward-regexp (if sgml-have-re-char-clases
+				 "^ *[-_.:[:alnum:]]+ *= ?"
+			       "^ *[-_.:A-Za-z0-9]+ *= ?") nil t)
       (goto-char (point-min))))
 
 
 ;;;; SGML mode: Hiding tags/attributes
 
 (defconst sgml-tag-regexp
-  "\\(</?>\\|</?[_A-Za-z][---_:A-Za-z0-9.]*\\(\\([^'\"></]\\|'[^']*'\\|\"[^\"]*\"\\)*\\)/?>?\\)")
+  (if sgml-have-re-char-clases
+      "\\(</?>\\|</?[_[:alpha:]][-_:[:alnum:].]*\\(\\([^'\"></]\\|'[^']*'\\|\"[^\"]*\"\\)*\\)/?>?\\)"
+    "\\(</?>\\|</?[_A-Za-z][-_:A-Za-z0-9.]*\\(\\([^'\"></]\\|'[^']*'\\|\"[^\"]*\"\\)*\\)/?>?\\)"))
 
 (defun sgml-operate-on-tags (action &optional attr-p)
   (let ((buffer-modified-p (buffer-modified-p))
@@ -1573,7 +1629,7 @@ value.  To abort edit kill buffer (\\[kill-buffer]) and remove window
 				      '(invisible nil)))
 	     (t (error "Invalid action: %s" action)))
 	    (incf tagcount)))
-      (set-buffer-modified-p buffer-modified-p))))
+      (sgml-restore-buffer-modified-p buffer-modified-p))))
 
 (defun sgml-hide-tags ()
   "Hide all tags in buffer."
@@ -1712,7 +1768,9 @@ elements with omitted end-tags."
   "Convert character after point into a character reference.
 If called with a numeric argument, convert a character reference back
 to a normal character.  If called from a program, set optional
-argument INVERT to non-nil."
+argument INVERT to non-nil.  If the function `decode-char' is defined,
+convert to and from Unicodes.  Otherwise will only work for ASCII or 8-bit
+characters in the current coding system."
   (interactive "*P")
   (cond
    (invert
@@ -1722,11 +1780,20 @@ argument INVERT to non-nil."
 					      (match-end 1)))))
       (delete-region (match-beginning 0)
 		     (match-end 0))
+      (if (fboundp 'decode-char)	; Emacs 21, Mule-UCS
+	  (setq c (decode-char 'ucs c))
+	;; Else have to assume 8-bit character.
+	(if (fboundp 'unibyte-char-to-multibyte) ; Emacs 20
+	    (setq c (unibyte-char-to-multibyte c))))
       (insert c)))
    ;; Convert character to &#nn;
    (t
     (let ((c (following-char)))
       (delete-char 1)
+      (if (fboundp 'encode-char)
+	  (setq c (encode-char c 'ucs))
+	(if (fboundp 'multibyte-char-to-unibyte)
+	    (setq c (multibyte-char-to-unibyte c))))
       (insert (format "&#%d;" c))))))
 
 (defun sgml-expand-entity-reference ()
@@ -1943,6 +2010,7 @@ If it is something else complete with ispell-complete-word."
     (when var
       (sgml-do-set-option var event))))
 
+;; Fixme: Use Customize for this.
 (defun sgml-do-set-option (var &optional event)
   (let ((type (sgml-variable-type var))
 	(val (symbol-value var)))
@@ -2048,13 +2116,16 @@ will reset the variable.")
         (s (sgml-element-model el))
         (found nil))
     (loop do
-          (dolist (tok (nconc (sgml-optional-tokens s)
-                              (sgml-required-tokens s)))
-            (unless (memq tok found)
-              ;; tok is optional here and not already found -- check that
-              ;; it would not make the content invalid
-              (when (sgml--add-before-p tok s c)
-                  (push tok found))))
+	  ;; Fixme: this test avoids an error when DTD-less, but it's
+	  ;; probably an inappropriate kludge.  -- fx
+          (when (not (eq s 'ANY))
+	    (dolist (tok (nconc (sgml-optional-tokens s)
+				(sgml-required-tokens s)))
+	      (unless (memq tok found)
+		;; tok is optional here and not already found -- check that
+		;; it would not make the content invalid
+		(when (sgml--add-before-p tok s c)
+                  (push tok found)))))
           while c do
           (setq s (sgml-element-pstate c))
           (setq c (sgml-element-next c)))
