@@ -1,5 +1,5 @@
 ;;; psgml.el --- SGML-editing mode with parsing support
-;; $Id: psgml.el,v 2.62 2002/02/15 15:32:10 lenst Exp $
+;; $Id: psgml.el,v 2.66 2002/08/13 10:51:25 lenst Exp $
 
 ;; Copyright (C) 1993-2002 Lennart Staflin
 ;; Copyright (C) 1992 Free Software Foundation, Inc.
@@ -35,7 +35,7 @@
 
 ;; - Identify structural errors (but it is not a validator)
 ;; - Menus for inserting tags with only the contextually valid tags
-;; - Edit attribute values in separate window with information about types 
+;; - Edit attribute values in separate window with information about types
 ;;   and defaults
 ;; - Hide attributes
 ;; - Fold elements
@@ -46,34 +46,42 @@
 
 ;; LIMITATIONS
 
-;; - only accepts the referece concrete syntax, though it does allow
+;; - only accepts the reference concrete syntax, though it does allow
 ;;   unlimited lengths on names
 
 
 ;;; Code:
 
-(defconst psgml-version "1.2.5"
+(defconst psgml-version "1.3.1"
   "Version of psgml package.")
 
 (defconst psgml-maintainer-address "lenst@lysator.liu.se")
 
-(require 'cl)
+(eval-when-compile (require 'cl))
 (require 'easymenu)
 
 (defvar sgml-debug nil)
 
-(defmacro sgml-debug (&rest x) 
+(defmacro sgml-debug (&rest x)
   (list 'if 'sgml-debug (cons 'sgml-log-message x)))
 
 
 ;;;; Variables
 
 (defvar sgml-mode-abbrev-table nil
-  "Abbrev table in use in sgml-mode.")
+  "Abbrev table in use in SGML mode.")
 (define-abbrev-table 'sgml-mode-abbrev-table ())
 
 (eval-and-compile
-  (defvar sgml-running-lucid (string-match "Lucid" emacs-version)))
+  (defconst sgml-running-lucid (string-match "Lucid" emacs-version))
+  (defconst sgml-have-re-char-clases (string-match "[[:alpha:]]" "x")
+    "Non-nil if this Emacs supports regexp character classes.
+E.g. `[-.[:alnum:]]'."))
+
+(defconst sgml-default-nonsticky (boundp 'text-property-default-nonsticky)
+  "Non-nil means use `text-property-default-nonsticky'. locally.
+Otherwise put explicit properties.")
+
 
 (defvar sgml-xml-p nil
   "Is this an XML document?")
@@ -96,44 +104,73 @@ after the start-tag. Useful on slow terminals if you find the end-tag after
 the cursor irritating." )
 
 (defvar sgml-doctype nil
-  "*If set, this should be the name of a file that contains the doctype
+  "*If non-nil the name of a file that contains the doctype declaration to use.
 declaration to use.
 Setting this variable automatically makes it local to the current buffer.")
 (put 'sgml-doctype 'sgml-type 'string)
 (make-variable-buffer-local 'sgml-doctype)
-
+  
 (defvar sgml-system-identifiers-are-preferred nil
-  "*If nil, PSGML will look up external entities by searching the catalogs
+  "*Controls lookup of external entities.
+nil means look up external entities by searching the catalogs
 in `sgml-local-catalogs' and `sgml-catalog-files' and only if the
-entity is not found in the catalogs will a given system identifier be
-used. If the variable is non-nil and a system identifier is given, the
-system identifier will be used for the entity. If no system identifier
-is given the catalogs will searched.")
+entity is not found in the catalogs use a given system identifier.
+Non-nil means use a system identifier for the entity if one is given.
+If no system identifier is given the catalogs will searched.")
 
 (defvar sgml-range-indicator-max-length 9
-  "*Maximum number of characters used from the first and last entry
+  "*Control of menu indicators.
+Maximum number of characters used from the first and last entry
 of a submenu to indicate the range of that menu.")
 
 (defvar sgml-default-doctype-name nil
-  "*Document type name to use if no document type declaration is present.")
+  "*If non-nil, document type name to use if no DTD is present.")
 (put 'sgml-default-doctype-name 'sgml-type 'string-or-nil)
 
-(defvar sgml-markup-faces '((start-tag 	. bold)
-			    (end-tag 	. bold)
-			    (comment 	. italic)
-			    (pi 	. bold)
-			    (sgml 	. bold)
-			    (doctype 	. bold)
-			    (entity 	. bold-italic)
-			    (shortref   . bold))
-  "*List of markup to face mappings.
+(defvar sgml-markup-faces
+  ;; Fixme: are the font-lock correspondences here the most appopriate
+  ;; ones?  I don't recall whence this set came.  -- fx
+  `((start-tag . ,(if (facep 'font-lock-function-name-face)
+		      'font-lock-function-name-face
+		    'bold))
+    (end-tag . ,(if (facep 'font-lock-function-name-face)
+		    'font-lock-function-name-face
+		  'bold))
+    (comment . ,(if (facep 'font-lock-comment-face)
+		    'font-lock-comment-face
+		  'bold))
+    (pi . ,(if (facep 'font-lock-type-face)
+	       'font-lock-type-face
+	     'bold))
+    (sgml . ,(if (facep 'font-lock-type-face)
+		 'font-lock-type-face
+	       'bold))
+    (doctype . ,(if (facep 'font-lock-keyword-face)
+		    'font-lock-keyword-face
+		  'bold))
+    (entity . ,(if (facep 'font-lock-string-face)
+		   'font-lock-string-face
+		 'bold))
+    (shortref . ,(if (facep 'font-lock-string-face)
+		     'font-lock-string-face
+		   'bold))
+    (ignored . ,(if (facep 'font-lock-constant-face)
+		    'font-lock-constant-face
+		  'default))
+    (ms-start . ,(if (facep 'font-lock-constant-face)
+		    'font-lock-constant-face
+		  'default))
+    (ms-end . ,(if (facep 'font-lock-constant-face)
+		    'font-lock-constant-face
+		  'default)))
+  "*Alist of markup to face mappings.
 Element are of the form (MARKUP-TYPE . FACE).
-Possible values for MARKUP-TYPE is:
+Possible values for MARKUP-TYPE are:
 comment	- comment declaration
 doctype	- doctype declaration
-end-tag 
+end-tag
 ignored	- ignored marked section
-ms-end	- marked section start, if not ignored 
+ms-end	- marked section start, if not ignored
 ms-start- marked section end, if not ignored
 pi	- processing instruction
 sgml	- SGML declaration
@@ -155,7 +192,8 @@ will be slower.")
 (put 'sgml-set-face 'sgml-desc "Set face of parsed markup")
 
 (defvar sgml-live-element-indicator nil
-  "*If non-nil, indicate current element in mode line.")
+  "*If non-nil, indicate current element in mode line.
+This may be slow.")
 
 (defvar sgml-auto-activate-dtd nil
   "*If non-nil, loading a sgml-file will automatically try to activate its DTD.
@@ -168,10 +206,10 @@ will be shown in the mode line.")
   "*If non-nil, ask about saving modified buffers before \\[sgml-validate] is run.")
 
 (defvar sgml-parent-document nil
-  "* Used when the current file is part of a bigger document.
+  "*How to handle the current file as part of a bigger document.
 
 The variable describes how the current file's content fit into the element
-hierarchy. The variable should have the form
+hierarchy.  The value should have the form
 
   (PARENT-FILE CONTEXT-ELEMENT* TOP-ELEMENT (HAS-SEEN-ELEMENT*)?)
 
@@ -179,16 +217,16 @@ PARENT-FILE	is a string, the name of the file containing the
 		document entity.
 CONTEXT-ELEMENT is a string, that is the name of an element type.
 		It can occur 0 or more times and is used to set up
-		exceptions and short reference map. Good candidates
+		exceptions and short reference map.  Good candidates
 		for these elements are the elements open when the
-		entity pointing to the current file is used. 
+		entity pointing to the current file is used.
 TOP-ELEMENT	is a string that is the name of the element type
-		of the top level element in the current file. The file
+		of the top level element in the current file.  The file
 		should contain one instance of this element, unless
-		the last \(lisp) element of sgml-parent-document is a
-		list. If it is a list, the top level of the file
-		should follow the content model of top-element. 
-HAS-SEEN-ELEMENT is a string that is the name of an element type. This
+		the last \(Lisp) element of `sgml-parent-document' is a
+		list.  If it is a list, the top level of the file
+		should follow the content model of top-element.
+HAS-SEEN-ELEMENT is a string that is the name of an element type.  This
 	        element is satisfied in the content model of top-element.
 
 Setting this variable automatically makes it local to the current buffer.")
@@ -205,7 +243,7 @@ transient-mark-mode must be on for the region to be tagged.")
 when adding end tag.")
 
 (defvar sgml-omittag t
-  "*Set to non-nil, if you use OMITTAG YES.
+  "*Non-nil means use OMITTAG YES.
 
 Setting this variable automatically makes it local to the current buffer.")
 
@@ -213,7 +251,7 @@ Setting this variable automatically makes it local to the current buffer.")
 (put 'sgml-omittag 'sgml-desc "OMITTAG")
 
 (defvar sgml-shorttag t
-  "*Set to non-nil, if you use SHORTTAG YES.
+  "*Non-nil means use SHORTTAG YES.
 
 Setting this variable automatically makes it local to the current buffer.")
 
@@ -221,7 +259,7 @@ Setting this variable automatically makes it local to the current buffer.")
 (put 'sgml-shorttag 'sgml-desc "SHORTTAG")
 
 (defvar sgml-namecase-general t
-  "*Set to non-nil, if you use NAMECASE GENERAL YES.
+  "*Non-nil means use NAMECASE GENERAL YES.
 
 Setting this variable automatically makes it local to the current buffer.")
 
@@ -235,8 +273,8 @@ Setting this variable automatically makes it local to the current buffer.")
 
 (defvar sgml-general-insert-case 'lower
   "*The case that will be used for general names in inserted markup.
-This can be the symbol `lower' or `upper'. Only effective if
-sgml-namecase-general is true.")
+This can be the symbol `lower' or `upper'.  Only effective if
+`sgml-namecase-general' is true.")
 (put 'sgml-general-insert-case 'sgml-type '(lower upper))
 
 (defvar sgml-entity-insert-case nil)
@@ -246,7 +284,7 @@ sgml-namecase-general is true.")
   "*Determines minimization of attributes inserted by edit-attributes.
 Actually two things are done
 1. If non-nil, omit attribute name, if attribute value is from a token group.
-2. If 'max, omit attributes with default value.
+2. If `max', omit attributes with default value.
 
 Setting this variable automatically makes it local to the current buffer.")
 
@@ -255,7 +293,7 @@ Setting this variable automatically makes it local to the current buffer.")
      '(("No" . nil) ("Yes" . t) ("Max" . max)))
 
 (defvar sgml-always-quote-attributes t
-  "*If non-nil, quote all attribute values inserted after finishing edit attributes.
+  "*Non-nil means quote all attribute values inserted after editing attributes.
 Setting this variable automatically makes it local to the current buffer.")
 
 (make-variable-buffer-local 'sgml-always-quote-attributes)
@@ -319,23 +357,23 @@ Setting this variable automatically makes it local to the current buffer.")
   (if (null cd-path)
       nil
     (let ((cd-sep ":")
-	  cd-list (cd-start 0) cd-colon)
+          cd-list (cd-start 0) cd-colon)
       (if (boundp 'path-separator)
-	  (setq cd-sep path-separator))
+          (setq cd-sep path-separator))
       (setq cd-path (concat cd-path cd-sep))
       (while (setq cd-colon (string-match cd-sep cd-path cd-start))
-	(setq cd-list
-	      (nconc cd-list
-		     (list (if (= cd-start cd-colon)
-			       nil
-			     (substitute-in-file-name
-			      (substring cd-path cd-start cd-colon))))))
-	(setq cd-start (+ cd-colon 1)))
+        (setq cd-list
+              (nconc cd-list
+                     (list (if (= cd-start cd-colon)
+                               nil
+                             (substitute-in-file-name
+                              (substring cd-path cd-start cd-colon))))))
+        (setq cd-start (+ cd-colon 1)))
       cd-list)))
 
 (defvar sgml-system-path (sgml-parse-colon-path
-			  (or (getenv "SGML_SEARCH_PATH")
-			      "."))
+			     (or (getenv "SGML_SEARCH_PATH")
+				 "."))
   "*List of directories used to look for system identifiers.")
 (put 'sgml-system-path 'sgml-type 'file-list)
 
@@ -349,9 +387,9 @@ the string by substitution of the whole public identifier for %P,
 owner for %O, public text class for %C, and public text description
 for %D.  The text class will be converted to lower case and the owner
 and description will be transliterated according to the variable
-sgml-public-transliterations.  If the file exists it will be the file
+`sgml-public-transliterations'.  If the file exists it will be the file
 used for the public identifier.  An element can also be a dotted pair
-(regexp . filename), the filename is a string treated as above, but
+\(regexp . filename), the filename is a string treated as above, but
 only if the regular expression, regexp, matches the public
 identifier.")
 (put 'sgml-public-map 'sgml-type 'list)
@@ -426,12 +464,12 @@ Example:
 (defvar sgml-custom-dtd nil
   "Menu entries to be added to the DTD menu.
 The value should be a list of entries to be added to the DTD menu.
-Every entry should be a list. The first element of the entry is a string
+Every entry should be a list.  The first element of the entry is a string
 used as the menu entry.  The second element is a string containing a
 doctype declaration (this can be nil if no doctype).  The rest of the
 list should be a list of variables and values.  For backward
 compatibility a single string instead of a variable is assigned to
-sgml-default-dtd-file.  All variables are made buffer local and are also
+`sgml-default-dtd-file'.  All variables are made buffer local and are also
 added to the buffers local variables list.
 
 Example:
@@ -452,8 +490,8 @@ Example:
 (put 'sgml-fixed 'face 'underline)	; Face of #FIXED "..."
 
 
-;;; sgmls is a free SGML parser available from
-;;; ftp.uu.net:pub/text-processing/sgml
+;;; nsgmls is a free SGML parser in the SP suite available from
+;;; ftp.jclark.com:pub/sp
 ;;; Its error messages can be parsed by next-error.
 ;;; The -s option suppresses output.
 
@@ -497,16 +535,19 @@ format control string instead of the defaults.")
 See `compilation-error-regexp-alist'.")
 
 (defvar sgml-declaration nil
-  "*If non-nil, this is the name of the SGML declaration file.")
+  "*If non-nil, the name of the SGML declaration file.")
 (put 'sgml-declaration 'sgml-type 'file-or-nil)
 
 (defvar sgml-xml-declaration nil
-  "*If non-nil, this is the name of the SGML declaration for XML files.")
+  "*If non-nil, the name of the SGML declaration for XML files.")
 (put 'sgml-xml-declaration 'sgml-type 'file-or-nil)
 
 (defvar sgml-mode-hook nil
   "A hook or list of hooks to be run when entering sgml-mode")
 
+(defvar sgml-mode-map nil
+  "Keymap for SGML mode")
+
 (defconst sgml-file-options
   '(
     sgml-omittag
@@ -564,8 +605,6 @@ See `compilation-error-regexp-alist'.")
 (defvar sgml-validate-command-history nil
   "The minibuffer history list for `sgml-validate''s COMMAND argument.")
 
-(defvar sgml-mode-map nil "Keymap for SGML mode")
-
 (defvar sgml-active-dtd-indicator nil
   "Displayed in the mode line")
 
@@ -590,7 +629,7 @@ See `compilation-error-regexp-alist'.")
   "Set the value of variable VAR to VAL in buffer and local variables list."
   (set (make-local-variable var) val)
   (save-excursion
-    (let ((prefix "") 
+    (let ((prefix "")
 	  (suffix "")
 	  (case-fold-search t))
       (goto-char (max (point-min) (- (point-max) 3000)))
@@ -640,7 +679,7 @@ See `compilation-error-regexp-alist'.")
 	   t))))
 
 (defun sgml-save-options ()
-  "Save user options for sgml-mode that have buffer local values."
+  "Save user options for SGML mode that have buffer local values."
   (interactive)
   (loop for var in sgml-file-options do
 	(when (sgml-valid-option var)
@@ -719,7 +758,7 @@ as that may change."
     end))
 
 
-;;;; SGML mode: indentation 
+;;;; SGML mode: indentation
 
 (defun sgml-indent-or-tab ()
   "Indent line in proper way for current major mode."
@@ -746,7 +785,7 @@ as that may change."
 	 'sgml-auto-activate-dtd
 	 'sgml-auto-insert-required-elements
 	 'sgml-balanced-tag-edit
-	 'sgml-catalog-files 
+	 'sgml-catalog-files
 	 'sgml-declaration
 	 'sgml-doctype
 	 'sgml-ecat-files
@@ -754,7 +793,7 @@ as that may change."
 	 'sgml-indent-step
 	 'sgml-leave-point-after-insert
 	 'sgml-live-element-indicator
-	 'sgml-local-catalogs 
+	 'sgml-local-catalogs
 	 'sgml-local-ecat-files
 	 'sgml-markup-faces
 	 'sgml-minimize-attributes
@@ -1042,7 +1081,7 @@ as that may change."
   nil)
 
 
-;;;; Post command hook 
+;;;; Post command hook
 
 (defvar sgml-auto-activate-dtd-tried nil)
 (make-variable-buffer-local 'sgml-auto-activate-dtd-tried)
@@ -1063,21 +1102,17 @@ actually only the state that persists between commands.")
 
 
 (defun sgml-command-post ()
-  (when (or (memq major-mode '(sgml-mode xml-mode))
-	    (fboundp 'make-local-hook))
-    ;; Explanation if make-local-hook is defined then this is called
-    ;; from a local hook and need not check major-mode.
-    (when (and (null sgml-buffer-parse-state)
-	       sgml-auto-activate-dtd
-	       (null sgml-auto-activate-dtd-tried)
-	       (not (zerop (buffer-size)))
-	       (looking-at ".*<"))
-      (setq sgml-auto-activate-dtd-tried t)
-      (ignore-errors
-        (sgml-need-dtd)
-        (sgml-fontify-buffer 0)))
-    (when sgml-buffer-parse-state
-      (sgml-update-display))))
+  (when (and (null sgml-buffer-parse-state)
+	     sgml-auto-activate-dtd
+	     (null sgml-auto-activate-dtd-tried)
+	     (not (zerop (buffer-size)))
+	     (looking-at ".*<"))
+    (setq sgml-auto-activate-dtd-tried t)
+    (ignore-errors
+     (sgml-need-dtd)
+     (sgml-fontify-buffer 0)))
+  (when sgml-buffer-parse-state
+    (sgml-update-display)))
 
 
 ;;;; SGML mode: major mode definition
@@ -1086,8 +1121,8 @@ actually only the state that persists between commands.")
 
 ;;;###autoload
 (defun sgml-mode ()
-  "Major mode for editing SGML.\\<sgml-mode-map>
-Makes > display the matching <.  Makes / display matching /.
+  "Major mode for editing SGML.
+\\<sgml-mode-map>Makes > display the matching <.  Makes / display matching /.
 Use \\[sgml-validate] to validate your document with an SGML parser.
 
 You can find information with:
@@ -1096,8 +1131,8 @@ You can find information with:
 
 Insert tags with completion of contextually valid tags with \\[sgml-insert-tag].
 End the current element with \\[sgml-insert-end-tag].  Insert an element (i.e.
-both start and end tag) with \\[sgml-insert-element].  Or tag a region with 
-\\[sgml-tag-region]. 
+both start and end tag) with \\[sgml-insert-element].  Or tag a region with
+\\[sgml-tag-region].
 
 To tag a region with the mouse, use transient mark mode or secondary selection.
 
@@ -1112,11 +1147,11 @@ Structure editing:
 
 Finding interesting positions
 \\[sgml-next-data-field]  Move forward to next point where data is allowed.
-\\[sgml-next-trouble-spot]  Move forward to next point where something is 
+\\[sgml-next-trouble-spot]  Move forward to next point where something is
 	amiss with the structure.
 
 Folding and unfolding
-\\[sgml-fold-element]  Fold the lines comprising the current element, leaving 
+\\[sgml-fold-element]  Fold the lines comprising the current element, leaving
 	the first line visible.
 \\[sgml-fold-subelement]  Fold the elements in the content of the current element.
 	Leaving the first line of every element visible.
@@ -1127,21 +1162,21 @@ User options:
 sgml-omittag  Set this to reflect OMITTAG in the SGML declaration.
 sgml-shorttag  Set this to reflect SHORTTAG in the SGML declaration.
 sgml-namecase-general  Set this to reflect NAMECASE GENERAL in the SGML declaration.
-sgml-auto-insert-required-elements  If non-nil, automatically insert required 
+sgml-auto-insert-required-elements  If non-nil, automatically insert required
 	elements in the content of an inserted element.
 sgml-omittag-transparent  If non-nil, will show legal tags inside elements
 	with omitable start tags and legal tags beyond omitable end tags.
-sgml-leave-point-after-insert  If non-nil, the point will remain after 
+sgml-leave-point-after-insert  If non-nil, the point will remain after
 	inserted tag(s).
-sgml-warn-about-undefined-elements  If non-nil, print a warning when a tag 
+sgml-warn-about-undefined-elements  If non-nil, print a warning when a tag
 	for a undefined element is found.
 sgml-max-menu-size  Max number of entries in Tags and Entities menus before
  	they are split into several panes.
-sgml-always-quote-attributes  If non-nil, quote all attribute values 
+sgml-always-quote-attributes  If non-nil, quote all attribute values
 	inserted after finishing edit attributes.
-sgml-minimize-attributes  Determines minimization of attributes inserted by 
+sgml-minimize-attributes  Determines minimization of attributes inserted by
 	edit-attributes.
-sgml-normalize-trims  If non-nil, sgml-normalize will trim off white space 
+sgml-normalize-trims  If non-nil, sgml-normalize will trim off white space
 	from end of element when adding end tag.
 sgml-indent-step  How much to increment indent for every element level.
 sgml-indent-data  If non-nil, indent in data/mixed context also.
@@ -1152,8 +1187,7 @@ sgml-offer-save  If non-nil, ask about saving modified buffers before
 		\\[sgml-validate] is run.
 
 All bindings:
-\\{sgml-mode-map}
-"
+\\{sgml-mode-map}"
   (interactive)
   (kill-all-local-variables)
   (setq sgml-xml-p nil)
@@ -1167,9 +1201,13 @@ All bindings:
   ;; immediately after a start tag or immediately before an end tag.
 
   (set (make-local-variable 'paragraph-separate)
-	"^[ \t\n]*$\\|\
+       (if sgml-have-re-char-clases
+           "^[ \t\n]*$\\|\
+^[ \t]*</?\\([_[:alpha:]]\\([-:._[:alnum:]= \t\n]\\|\
+\"[^\"]*\"\\|'[^']*'\\)*\\)?>$"
+         "^[ \t\n]*$\\|\
 ^[ \t]*</?\\([_A-Za-z]\\([-:._A-Za-z0-9= \t\n]\\|\
-\"[^\"]*\"\\|'[^']*'\\)*\\)?>$")
+\"[^\"]*\"\\|'[^']*'\\)*\\)?>$"))
   (set (make-local-variable 'paragraph-start)
        paragraph-separate)
 
@@ -1189,29 +1227,33 @@ All bindings:
   (make-local-variable 'indent-line-function)
   (setq indent-line-function 'sgml-indent-line)
   (make-local-variable 'mode-line-format)
-  ;; Modify mode-line-format with susbt (sugested by wing)
-  (setq mode-line-format
- 	(subst '("" mode-name sgml-active-dtd-indicator) 'mode-name
- 	       mode-line-format))
+  (if (featurep 'xemacs)
+      ;; Modify mode-line-format with susbt (sugested by wing)
+      ;; Apart from requiring CL at runtime, this doesn't work in Emacs
+      ;; 21.  It's the sort of thing which-func is supposed to do...
+      (setq mode-line-format
+	    (subst '("" mode-name sgml-active-dtd-indicator) 'mode-name
+		   mode-line-format))
+    (set (make-local-variable 'which-func-format) 'sgml-active-dtd-indicator))
   (make-local-variable 'sgml-default-dtd-file)
   (when (setq sgml-default-dtd-file (sgml-default-dtd-file))
     (unless (file-exists-p sgml-default-dtd-file)
       (setq sgml-default-dtd-file nil)))
-  (cond ((fboundp 'make-local-hook)
-	 ;; emacs>= 19.29
-	 (make-local-hook 'post-command-hook)
-	 (add-hook 'post-command-hook 'sgml-command-post 'append 'local)
-	 (unless sgml-running-lucid 
-	   ;; XEmacs 20.4 doesn't handle local activate-menubar-hook
-	   ;; it tries to call the function `t' when using the menubar
-	   (make-local-hook 'activate-menubar-hook))
-         (add-hook 'activate-menubar-hook 'sgml-update-all-options-menus
-		   nil 'local))
-	(t
-	 ;; emacs< 19.29
-	 (add-hook 'post-command-hook 'sgml-command-post 'append)
-         (add-hook 'menu-bar-update-hook 'sgml-update-all-options-menus)
-         ))
+;;; This doesn't DTRT with Emacs 21.1 newcomment -- intermediate lines
+;;; are prefixed by `!--'.  -- fx
+;;;   (set (make-local-variable 'comment-style) 'multi-line)
+  (when sgml-default-nonsticky
+    (make-local-variable 'text-property-default-nonsticky)
+    ;; see `sgml-set-face-for':
+    (add-to-list 'text-property-default-nonsticky '(face . t)))
+  (make-local-hook 'post-command-hook)
+  (add-hook 'post-command-hook 'sgml-command-post 'append 'local)
+  (unless sgml-running-lucid
+    ;; XEmacs 20.4 doesn't handle local activate-menubar-hook
+    ;; it tries to call the function `t' when using the menubar
+    (make-local-hook 'activate-menubar-hook))
+  (add-hook 'activate-menubar-hook 'sgml-update-all-options-menus
+	    nil 'local)
   (run-hooks 'text-mode-hook 'sgml-mode-hook)
   (easy-menu-add sgml-main-menu)
   (easy-menu-add sgml-modify-menu)
@@ -1221,10 +1263,23 @@ All bindings:
   (easy-menu-add sgml-dtd-menu)
   (sgml-build-custom-menus))
 
-
+;; It would be nice to generalize the `auto-mode-interpreter-regexp'
+;; machinery so that we could select xml-mode on the basis of the
+;; leading xml PI.  -- fx
 
 ;;;###autoload
 (define-derived-mode xml-mode sgml-mode "XML"
+  "Major mode for editing XML, specialized from SGML mode.
+Sets various variables appropriately for XML.
+
+Can be used without a DTD.  In that case, warnings about undefined
+elements and entities are suppressed and various commands' behaviour
+is modified to account for the lack of information.  For instance, the
+element names offered for selection or completion are those in the
+parse of the document, but other names may be entered.
+
+Note that without a DTD, indenting lines will only work if
+`sgml-indent-data' is non-nil."
   (setq sgml-xml-p t)
   ;; XML-friendly settings
   (setq sgml-omittag nil)
@@ -1257,7 +1312,9 @@ All bindings:
     0))
 
 (defconst sgml-start-tag-regex
-  "<[_A-Za-z]\\([-:.A-Za-z0-9= \n\t]\\|\"[^\"]*\"\\|'[^']*'\\)*"
+  (if sgml-have-re-char-clases
+      "<[_[:alpha:]]\\([-:.[:alnum:]= \n\t]\\|\"[^\"]*\"\\|'[^']*'\\)*"
+    "<[_A-Za-z]\\([-:.A-Za-z0-9= \n\t]\\|\"[^\"]*\"\\|'[^']*'\\)*")
   "Regular expression that matches a non-empty start tag.
 Any terminating > or / is not matched.")
 
@@ -1272,7 +1329,7 @@ Any terminating > or / is not matched.")
   (modify-syntax-entry ?- "_ 1234" sgml-mode-markup-syntax-table)
   (modify-syntax-entry ?\' "\"" sgml-mode-markup-syntax-table))
 
-(defconst sgml-angle-distance 4000
+(defvar sgml-angle-distance 4000
   "*If non-nil, is the maximum distance to search for matching <.")
 
 (defun sgml-close-angle (arg)
@@ -1292,8 +1349,11 @@ Any terminating > or / is not matched.")
 	    (and (> (- (point) (point-min)) 3)
 		 (eq (char-after (- (point) 2)) ?\])
 		 (eq (char-after (- (point) 3)) ?\])
-		 (re-search-backward "<!\\[\\(-?[A-Za-z0-9. \t\n&;]\\|\
+		 (re-search-backward (if sgml-have-re-char-clases
+					 "<!\\[\\(-?[[:alnum:]. \t\n&;]\\|\
 --\\([^-]\\|-[^-]\\)*--\\)*\\["
+				       "<!\\[\\(-?[A-Za-z0-9. \t\n&;]\\|\
+--\\([^-]\\|-[^-]\\)*--\\)*\\[")
 				     (point-min)
 				     t)
 		 (let ((msspos (point)))
@@ -1320,12 +1380,16 @@ Any terminating > or / is not matched.")
 		   (or
 		    ;; Check that it's a valid delimiter in context.
 		    (not (looking-at
-			  "<\\(\\?\\|/?[A-Za-z>]\\|!\\([[A-Za-z]\\|--\\)\\)"))
+			  (if sgml-have-re-char-clases
+			      "<\\(\\?\\|/?[[:alpha:]>]\\|!\\([[[:alpha:]]\\|--\\)\\)"
+			    "<\\(\\?\\|/?[A-Za-z>]\\|!\\([[A-Za-z]\\|--\\)\\)")))
 		    ;; Check that it's not a net-enabling start tag
 		    ;; nor an unclosed start-tag.
 		    (looking-at (concat sgml-start-tag-regex "[/<]"))
 		    ;; Nor an unclosed end-tag.
-		    (looking-at "</[A-Za-z][-:.A-Za-z0-9]*[ \t]*<"))
+		    (looking-at (if sgml-have-re-char-clases
+				    "</[[:alpha:]][-:.[:alnum:]]*[ \t]*<"
+				  "</[A-Za-z][-:.A-Za-z0-9]*[ \t]*<")))
 		   (setq blinkpos nil)))
 	    (if blinkpos
 		()
@@ -1348,7 +1412,7 @@ Any terminating > or / is not matched.")
 
 ;;; I doubt that null end tags are used much for large elements,
 ;;; so use a small distance here.
-(defconst sgml-slash-distance 1000
+(defvar sgml-slash-distance 1000
   "*If non-nil, is the maximum distance to search for matching /.")
 
 (defun sgml-slash (arg)
@@ -1436,7 +1500,10 @@ and move to the line in the SGML document that caused it."
 		    nil
 		    sgml-validate-error-regexps))
 
-
+(defalias 'sgml-restore-buffer-modified-p
+  (if (fboundp 'restore-buffer-modified-p)
+      'restore-buffer-modified-p	; doesn't update mode line
+    'set-buffer-modified-p))
 
 ;;;; Autoloads and hooks
 
@@ -1469,7 +1536,7 @@ With implied tags this is ambiguous." t)
 (autoload 'sgml-mark-element "psgml-edit" "Set mark after next element." t)
 (autoload 'sgml-mark-current-element "psgml-edit" "Set mark at end of current element, and leave point before current element." t)
 (autoload 'sgml-change-element-name "psgml-edit" "Replace the name of the current element with a new name.
-Eventual attributes of the current element will be translated if 
+Eventual attributes of the current element will be translated if
 possible." t)
 (autoload 'sgml-untag-element "psgml-edit" "Remove tags from current element." t)
 (autoload 'sgml-kill-markup "psgml-edit" "Kill next tag, markup declaration or process instruction." t)
@@ -1510,7 +1577,7 @@ is left after the inserted tag(s), unless the element has some required
 content.  If sgml-leave-point-after-insert is nil the point is left
 after the first tag inserted." t)
 (autoload 'sgml-element-menu "psgml-edit" "Pop up a menu with valid elements and insert choice.
-If sgml-leave-point-after-insert is nil the point is left after the first 
+If sgml-leave-point-after-insert is nil the point is left after the first
 tag inserted." t)
 (autoload 'sgml-start-tag-menu "psgml-edit" "Pop up a menu with valid start-tags and insert choice." t)
 (autoload 'sgml-end-tag-menu "psgml-edit" "Pop up a menu with valid end-tags and insert choice." t)
@@ -1553,7 +1620,7 @@ argument INVERT to non-nil." t)
 (autoload 'sgml-complete "psgml-edit" "Complete the word/tag/entity before point.
 If it is a tag (starts with < or </) complete with valid tags.
 If it is an entity (starts with &) complete with declared entities.
-If it is a markup declaration (starts with <!) complete with markup 
+If it is a markup declaration (starts with <!) complete with markup
 declaration names.
 If it is something else complete with ispell-complete-word." t)
 (autoload 'sgml-file-options-menu "psgml-edit" nil t)
@@ -1579,11 +1646,5 @@ otherwise it will be added at the first legal position." t)
 (provide 'psgml)
 (provide 'sgml-mode)
 
-(eval-and-compile
-  (cond
-   (sgml-running-lucid
-    (require 'psgml-lucid))
-   (t
-    (require 'psgml-other))))
 
 ;;; psgml.el ends here
